function highest_row_complete = ...
    grow_nucleated_tube_distribution_with_bad_tiles(hours,seed_conc,num_tubes, ...
				     bad_tile_fraction,delta_G)

% We'll start with a very simple model that assumes that if a bad
% tile is attached, the nanotube can't grow at all, otherwise it
% grows normally.

% Assumes temp in C, tile conc is in nM.  Calculates growth rates
% in rows / second assuming that 
% k_on in /M/s
if nargin < 5
  %delta_H = -102.4;
  delta_G = -10.9;
end
if nargin < 1
  hours = 10;
  seed_conc = 40e-12;
  num_tubes = 1000;
  bad_tile_fraction = 0.3;
  %delta_H = -102.9;
  delta_G = -10.9;
end

  time_limit = 3600*hours;
avagadros_number = 6.022e23;
molecules_of_seeds_per_liter = seed_conc*avagadros_number;
reaction_volume = num_tubes/molecules_of_seeds_per_liter;

orig_tile_conc = 40e-9;
orig_good_tile_conc = orig_tile_conc*(1-bad_tile_fraction);
orig_bad_tile_conc = orig_tile_conc*bad_tile_fraction;
good_tiles_in_volume = orig_good_tile_conc*reaction_volume* ...
    avagadros_number;
bad_tiles_in_volume = orig_bad_tile_conc*reaction_volume*avagadros_number;
num_tile_types = 2;

good_tile_conc = orig_good_tile_conc;
bad_tile_conc = orig_bad_tile_conc;

debug = 0;

k_on = 5.7e5



temp = 32;

% ignore delta G for one bond at first.  Use delta H and S from zz
% paper (kcal/mol and kcal/mol/K)
%delta_H = -102.4;
delta_S = -0.300;

% Or from hariadi 2012
%delta_H = -85.8;
%delta_S = 0.246;


T = temp + 273;
%delta_G = delta_H - T*delta_S
R = 0.00198;

k_off = k_on * exp(delta_G/(R*T))

dae_e_repeat_unit = 14.3;
% 100 microns
rows_maximum= ceil(100000/14.3);
time = 0;


highest_row_complete = ones(num_tubes,1);
highest_row_empty = ones(num_tubes,1) + 1;

% in tiles
%tube_width = 12;
tube_width = 6;

filled_in = zeros(num_tubes,tube_width,rows_maximum+tube_width);
filled_in(:,:,1) = 1;

% On the tube lattice if the row is even
% below and above neighbors are x,x+1
% if the row is odd
% below and above neighbors are x-1,x

growth_site_rows = zeros(num_tubes,2*tube_width) + 2;
growth_site_tile_nums = zeros(num_tubes,2*tube_width);
removal_site_rows = zeros(num_tubes,2*tube_width) + 2;
removal_site_tile_nums = zeros(num_tubes,2*tube_width);

num_growth_sites = zeros(num_tubes,1) + tube_width;
num_removal_sites = zeros(num_tubes,1) ;
bad_tile_attached = zeros(num_tubes,1);

for i=1:tube_width
  growth_site_tile_nums(:,i) = zeros(num_tubes,1) + i;
  removal_site_tile_nums(:,i) = zeros(num_tubes,1) + i;
end
growth_site_rows = ones(num_tubes,tube_width) + 1;

a = (time < time_limit)
size(highest_row_complete)
b = min(highest_row_complete') < rows_maximum
while (time < time_limit) && (min(highest_row_complete) < rows_maximum)
  % Figure out where tiles could be added or removed
  total_growth_sites = sum(num_growth_sites.*(1- ...
					      bad_tile_attached));

  total_removal_sites = sum(num_removal_sites.*(1-bad_tile_attached)) ...
			    + sum(bad_tile_attached);
  total_on_rate = k_on*(good_tile_conc+bad_tile_conc)*total_growth_sites;
  total_off_rate = k_off*total_removal_sites;
  total_rate = total_on_rate + total_off_rate;
  which_rate = random('Uniform',0,total_rate);
  r_2 = random('Uniform',0,1);
  time = time + 1/total_rate * log(1/r_2);  
  if (which_rate < total_on_rate)
    %fprintf(1,'on event\n');
    % on event
    % Figure out if we are attaching a good tile or a bad tile
    which_site = ceil(which_rate/(total_on_rate/ ...
				  total_growth_sites));
    total_sites_seen = 0;
    for q=1:num_tubes
      if (bad_tile_attached(q) == 0)
	total_sites_seen = total_sites_seen + num_growth_sites(q);
	if which_site <= total_sites_seen 
	  break;
	end
      end
    end
    if rand() < bad_tile_conc / (bad_tile_conc  + good_tile_conc)
      % Bad tile attachment
      bad_tile_attached(q) = 1;
      bad_tile_conc = bad_tile_conc - orig_bad_tile_conc/(bad_tiles_in_volume*num_tile_types);
    else
      which_site_for_tube = total_sites_seen - which_site  + 1;
      

      if debug
	fprintf(1,'adding a site to tube %d site %d out of %f,%f, circum pos %d, row %d\n',...
		q,which_site_for_tube,which_rate,total_on_rate,...
		growth_site_tile_nums(q,which_site_for_tube),...
		growth_site_rows(q,which_site_for_tube));
      end
      filled_in(q,growth_site_tile_nums(q,which_site_for_tube), ...
		growth_site_rows(q,which_site_for_tube)) = 1;
      good_tile_conc = good_tile_conc - orig_good_tile_conc/(good_tiles_in_volume*num_tile_types);
      % Check whether this on event completes a tile row
      %if (min(filled_in(q,:,growth_site_rows(which_site_for_tube))) > 0)
      % If so, might it allow us to move up the highest_row_complete
      % counter?
      % It will if every row between the current highest complete
      % one plus one and this one is complete
      %      'filled in'
      %pause
      %can_move_up_complete = 1;
      %      fprintf(1,'checking rows %d to %d\n',highest_row_complete+1, ...
      %	      growth_site_rows(which_site));
      %pause;
      %for j=(highest_row_complete(q)+1):growth_site_rows(q,which_site_for_tube)
      %if min(filled_in(q,:,j)) == 0
      % can_move_up_complete = 0;
      % break;
      %end
      %end
      %fprintf('can move up complete check was %d.\n',can_move_up_complete);
      %if can_move_up_complete
      %highest_row_complete(q) = ...
      %    growth_site_rows(q,which_site_for_tube);
      % Might the rows beyond this one also have been
      % complete?  If so, we should continue to increment the
      % highest_row_complete up to the last complete row
      %pause
      %while min(filled_in(q,:,highest_row_complete(q)+1)) > 0
      %  highest_row_complete(q) = highest_row_complete(q) + 1;
      %end
      %  end
      %end
      while min(filled_in(q,:,highest_row_complete(q)+1)) > 0
	highest_row_complete(q) = highest_row_complete(q) + 1;
      end
      % Check whether this on event initiates a new tile row
      % If so, it might allows us to move up the highest_row_empty
      % counter
      if (sum(filled_in(q,:,growth_site_rows(q,which_site_for_tube))) == 1)
	highest_row_empty(q) = growth_site_rows(q,which_site_for_tube) + 1;
      end
    end
  else
    which_rate = which_rate - total_on_rate;
    which_site = ceil(which_rate/k_off);
    total_sites_seen = 0;
    for q=1:num_tubes
      if (bad_tile_attached(q))
	% Only the bad tile can be detached once it is attached
	total_sites_seen = total_sites_seen + 1;
      else
	total_sites_seen = total_sites_seen + num_removal_sites(q);
      end
      if which_site <= total_sites_seen 
	break;
      end
    end
    if bad_tile_attached(q)
      bad_tile_attached(q) = 0;
      bad_tile_conc = bad_tile_conc + orig_bad_tile_conc/(bad_tiles_in_volume*num_tile_types);    
    else
      which_site_for_tube = total_sites_seen - which_site + 1;
      filled_in(q,removal_site_tile_nums(q,which_site_for_tube), ...
		removal_site_rows(q,which_site_for_tube)) = 0;    
      good_tile_conc = good_tile_conc + orig_good_tile_conc/(good_tiles_in_volume*num_tile_types);    
      if debug
	fprintf(1,'removing a site from tube %d site %d out of %f,%f, circum pos %d, row %d\n',...
		q,which_site_for_tube,which_rate,total_off_rate,...
		removal_site_tile_nums(q,which_site_for_tube),...
		removal_site_rows(q,which_site_for_tube));    
      end
      % Did this site make a row empty now?   If so, we need to
      % change the highest_row_empty counter
      %if sum(filled_in(q,:,removal_site_rows(q,which_site_for_tube))) ...
      %	   == 0
      %'emptying'
      %  highest_row_empty(q) = removal_site_rows(q,which_site_for_tube);
      %end
      % Did we uncomplete a row?  If so we may need to move back the
      % complete marker to the previous row
      %if sum(filled_in(q,:,removal_site_rows(q,which_site_for_tube))) == ...
      %	 (tube_width - 1)
      %  highest_row_complete(q) = min(highest_row_complete(q), ...
      %				 removal_site_rows(q,which_site_for_tube) - 1);
      %   end
      while highest_row_complete(q) > 1 && ...
	    min(filled_in(q,:,highest_row_complete(q)+1)) == 0
	highest_row_complete(q) = highest_row_complete(q) - 1;
      end
    end
  end
  if bad_tile_attached(q) == 0
    % Update the "which rows filled" arrays for the assembly we
    % updated
    num_growth_sites(q) = 0;
    num_removal_sites(q) = 0;
    for i=1:tube_width
      for j=(highest_row_complete(q)+1):highest_row_empty(q)
	% This is a growth site if it is empty and if it has at
        % least two filled neighbors.  
	if ((filled_in(q,i,j) == 0) && number_of_neighbors(q,i,j,filled_in,tube_width) ...
	    == 2)
	  num_growth_sites(q) = num_growth_sites(q) + 1;
	  growth_site_tile_nums(q,num_growth_sites(q)) = i;
	  growth_site_rows(q,num_growth_sites(q)) = j;
	end
	if ((filled_in(q,i,j) > 0) && number_of_neighbors(q,i,j,filled_in,tube_width) == 2)
	  num_removal_sites(q) = num_removal_sites(q) + 1;
	  removal_site_tile_nums(q,num_removal_sites(q)) = i;
	  removal_site_rows(q,num_removal_sites(q)) = j;
	end
      end
    end
  end
  if debug 
    fprintf('num growth sites: ');
    num_growth_sites'
    fprintf('num removal sites: ');
    num_removal_sites'
    fprintf('removal site rows: ')
    removal_site_rows(q,:)
    fprintf('removal site tile nums: ')
    removal_site_tile_nums(q,:)
    fprintf(1,['Tube %d.  There are %d growth sites, %d removal sites.  '...
	       'highest row complete is %d, highest row empty is %d.\n'],...
	    q,num_growth_sites(q), num_removal_sites(q), ...
	    highest_row_complete(q), highest_row_empty(q));
    fprintf(1,'Filled in rows are \n');
  end
  if debug
    for j=(highest_row_complete(q)+1):highest_row_empty(q)
      if (mod(j,2) == 0)
	fprintf(1,' ');
      end
      for i=1:tube_width
        fprintf(1,'%d ',filled_in(q,i,j));
      end
      fprintf(1,'\n');
    end
    fprintf(1,'\n\n');
  end
  %for q=1:num_tubes
  %  while min(filled_in(q,:,highest_row_complete(q)+1)) > 0
  %    highest_row_complete(q) = highest_row_complete(q) + 1;
  %    fprintf('Found bad row -- pausing');
  %    pause;
  %  end
%end

  if rand() > 0.999
    figure(1)
    %long_enough_tubes = find(highest_row_complete*14.3/156 > 2.5);
    %hist(highest_row_complete(long_enough_tubes)*14.3/156,3:0.5:60)    
    hist(highest_row_complete*14.3/1000,0:0.5:15)    
    fprintf('number of blocked tiles is %d, good tile conc is %e, bad tile conc is %e.\n', ...
	    sum(bad_tile_attached),good_tile_conc,bad_tile_conc);
    fprintf('on rate is %f, off rate is %f.\n',total_on_rate, ...
	    total_off_rate);
    fprintf('on sites are %d, off sites are %d.\n',total_growth_sites,total_removal_sites);
    axis([0 15 0 num_tubes])
    title(['time is ' num2str(time) ' seconds']);
  end
  %figure(1)
  %imagesc(filled_in)
end


figure(1)
%long_enough_tubes = find(highest_row_complete*14.3/156 > 2.5);
%hist(highest_row_complete(long_enough_tubes)*14.3/156,3:0.5:60)
hist(highest_row_complete*14.3/156,0:0.5:60)    
title(['Nanotube length dist. after ' num2str(time/3600) ' hours'],...
'FontSize',20);
xlabel('Length (pixels)','FontSize',18);
ylabel('Number of tubes','FontSize',18);
set(gca,'FontSize',18);

%for k=1:num_tubes
%  fprintf('Tube %d, %d rows.\n',k,highest_row_complete(k));
%  for j=highest_row_complete(k):highest_row_empty(k)
%    if (mod(j,2) == 0)
%      fprintf(1,' ');
%    end
%    for i=1:tube_width
%      fprintf(1,'%d ',filled_in(k,i,j));
%    end
%    fprintf(1,'\n');
%  end
%  fprintf(1,'\n\n');
%end

function neighbors = number_of_neighbors(tube,circle_pos,row,filled_in,tube_width)
  % On the tube lattice ..
  if mod(circle_pos-1,tube_width) == 0
    mod_minus = tube_width;
  else
    mod_minus = mod(circle_pos-1,tube_width);
  end
  if mod(circle_pos+1,tube_width) == 0
    mod_plus = tube_width;
  else
    mod_plus = mod(circle_pos+1,tube_width);
  end
  %fprintf(1,'circle pos is %d, minus is %d, plus is %d.\n',circle_pos,mod_minus,mod_plus);
  if (mod(row,2)) 
    % if the row is odd
    % below and above neighbors are x-1,x
    neighbors = filled_in(tube,mod_minus,row+1) + ...
	filled_in(tube,circle_pos,row+1) ...
	+ filled_in(tube,mod_minus,row-1) + ...
	filled_in(tube,circle_pos,row-1);
  else
    % if the row is even
    % below and above neighbors are x,x+1
    neighbors = filled_in(tube,mod_plus,row+1) + ...
	filled_in(tube,circle_pos,row+1) + ...
	filled_in(tube,mod_plus,row-1) + ...
	filled_in(tube,circle_pos,row-1);
  end
