function highest_row_complete = ...
    grow_nucleated_tube_distribution_with_unnuc_seed_barrier(hours,seed_conc,num_tubes, ...
				     k_join,k_per_row_scission,k_off_one)


% Assumes temp in C, tile conc is in nM.  Calculates growth rates
% in rows / second assuming that 
% k_on in /M/s
if nargin < 5
  k_join = 0;
  k_per_row_scission = 0;
end
if nargin < 1
  hours = 10;
  seed_conc = 40e-12;
  num_tubes = 1000;
  k_join = 35000
  %k_join=5.7e6;
  delta_G = -10.9
  R = 0.00198;
  T = 273+32;
  k_per_row_scission = 5.7e5*exp(delta_G*1.75/(R*T))
  k_off_one = 5.7e5*exp(delta_G*1.25/(R*T))
end


% /M/s, from PNAS 2007
%k_join = 35000;
%k_join = 5e6;



time_limit = 3600*hours;
avagadros_number = 6.022e23;
molecules_of_seeds_per_liter = seed_conc*avagadros_number;
reaction_volume = num_tubes/molecules_of_seeds_per_liter;

orig_tile_conc = 40e-9;
tiles_in_volume = orig_tile_conc*reaction_volume*avagadros_number;
num_tile_types = 2;

tile_conc = orig_tile_conc;

debug = 0;

k_on = 5.7e5;

 % Simple linear fit to Abdul's data
% macro_nucleation_rate = 1.5e-18; 
% Least squares fit to model that includes joining
macro_nucleation_rate = 2.2e-18;



avagadros_number = 6.022e23;
molecules_of_seeds_per_liter = seed_conc*avagadros_number;
reaction_volume = num_tubes/molecules_of_seeds_per_liter
% In molecules / s
%nucleation_rate = macro_nucleation_rate * avagadros_number*reaction_volume
nucleation_rate = macro_nucleation_rate * avagadros_number*reaction_volume*100

temp = 32;
tile_med_on = k_on*tile_conc;

% ignore delta G for one bond at first.  Use delta H and S from zz
% paper (kcal/mol and kcal/mol/K)
%delta_H = -102.4;
delta_S = -0.300;

% Or from hariadi 2012
%delta_H = -85.8;
%delta_S = 0.246;


T = temp + 273;
%delta_G = delta_H - T*delta_S
delta_G = -10.9;
R = 0.00198;

k_off = k_on * exp(delta_G/(R*T));

dae_e_repeat_unit = 14.3;
% 10 microns
rows_maximum= ceil(15000/14.3);
time = 0;
max_unnuc_tubes =26000;
num_unnuc_tubes = 0;

highest_row_complete = ones(num_tubes,1);
highest_row_empty = ones(num_tubes,1) + 1;
highest_row_complete_unnuc = ones(max_unnuc_tubes,1);
highest_row_empty_unnuc = ones(max_unnuc_tubes,1) + 1;


% in tiles
%tube_width = 12;
tube_width = 6;

filled_in = zeros(num_tubes,tube_width,rows_maximum+tube_width);
filled_in(:,:,1) = 1;
filled_in_unnuc = zeros(max_unnuc_tubes,tube_width,rows_maximum+tube_width);
filled_in_unnuc(:,:,1) = 1;


% On the tube lattice if the row is even
% below and above neighbors are x,x+1
% if the row is odd
% below and above neighbors are x-1,x

growth_site_rows = zeros(num_tubes,2*tube_width) + 2;
growth_site_tile_nums = zeros(num_tubes,2*tube_width);
removal_site_rows = zeros(num_tubes,2*tube_width) + 2;
removal_site_tile_nums = zeros(num_tubes,2*tube_width);

num_growth_sites = zeros(num_tubes,1) + tube_width;
num_removal_sites = zeros(num_tubes,1) ;
num_removal_sites_first_row = zeros(num_tubes,1) ;

growth_site_rows_unnuc = zeros(max_unnuc_tubes,2*tube_width) + 2;
growth_site_tile_nums_unnuc = zeros(max_unnuc_tubes,2*tube_width);
removal_site_rows_unnuc = zeros(max_unnuc_tubes,2*tube_width) + 2;
removal_site_tile_nums_unnuc = zeros(max_unnuc_tubes,2*tube_width);

num_growth_sites_unnuc = zeros(max_unnuc_tubes,1) + tube_width;
num_removal_sites_unnuc = zeros(max_unnuc_tubes,1) ;


for i=1:tube_width

  growth_site_tile_nums(:,i) = zeros(num_tubes,1) + i;
  removal_site_tile_nums(:,i) = zeros(num_tubes,1) + i;
  growth_site_tile_nums_unnuc(:,i) = zeros(max_unnuc_tubes,1) + i;
  removal_site_tile_nums_unnuc(:,i) = zeros(max_unnuc_tubes,1) + i;
end

num_unnuc_tubes = 0;

%a = (time < time_limit)
%size(highest_row_complete)
%b = min(highest_row_complete') < rows_maximum
while (time < time_limit) && (min(highest_row_complete) < rows_maximum)
  % Figure out where tiles could be added or removed
  if num_unnuc_tubes > 0
    total_growth_sites = sum(num_growth_sites) + ...
	sum(num_growth_sites_unnuc(1:num_unnuc_tubes));
    total_removal_sites = sum(num_removal_sites) + sum(num_removal_sites_unnuc(1:num_unnuc_tubes));
  else
    total_growth_sites = sum(num_growth_sites);
    total_removal_sites = sum(num_removal_sites);
  end

  total_on_rate = k_on*tile_conc*total_growth_sites;
  if debug
    fprintf(['total is %d addition sites, %d removal sites, %d' ...
	     ' removal sites first row.\n'],sum(num_growth_sites),...
	    sum(num_removal_sites), ...
	    sum(num_removal_sites_first_row));
  end
  total_off_rate = k_off*total_removal_sites + (k_off_one - k_off) ...
      * sum(num_removal_sites_first_row);
  seeded_scission_rate = k_per_row_scission*...
      (sum(highest_row_complete) - num_tubes);
  unnuc_tube_conc = num_unnuc_tubes / (reaction_volume * avagadros_number);
  if num_unnuc_tubes > 0
    seed_unseed_joining_rate = k_join * num_unnuc_tubes * ...
	seed_conc;
    unseed_unseed_joining_rate = k_join * unnuc_tube_conc * ...
	(num_unnuc_tubes - 1);
    unseeded_scission_rate = k_per_row_scission * (sum(highest_row_complete_unnuc(1:num_unnuc_tubes))-num_unnuc_tubes);
  else
    seed_unseed_joining_rate = 0;
    unseed_unseed_joining_rate = 0;
    unseeded_scission_rate = 0;
  end
  total_rate = total_on_rate + total_off_rate + nucleation_rate + ...
      seed_unseed_joining_rate + unseed_unseed_joining_rate + ...
      seeded_scission_rate + unseeded_scission_rate;
  which_rate = random('Uniform',0,total_rate);
  r_2 = random('Uniform',0,1);
  time = time + 1/total_rate * log(1/r_2);  
  if (which_rate < total_on_rate)
    %fprintf(1,'on event\n');
    % on event
    which_site = ceil(which_rate/(total_on_rate/ ...
				  total_growth_sites));
    total_sites_seen = 0;
    nuc_tube = 1;
    for q=1:num_tubes
      total_sites_seen = total_sites_seen + num_growth_sites(q);
      if which_site <= total_sites_seen 
	break;
      end
    end
    if (q == num_tubes) && which_site > sum(num_growth_sites)
       nuc_tube = 0;
      for r=1:num_unnuc_tubes
	total_sites_seen = total_sites_seen + num_growth_sites_unnuc(r);
	if which_site <= total_sites_seen 
	  break;
	end
      end
    end
    which_site_for_tube = total_sites_seen - which_site  + 1;
    if debug
      fprintf(1,'adding a site to tube %d site %d out of %f,%f, circum pos %d, row %d\n',...
	      q,which_site_for_tube,which_rate,total_on_rate,...
	      growth_site_tile_nums(q,which_site_for_tube),...
	      growth_site_rows(q,which_site_for_tube));
    end
    if nuc_tube
      filled_in(q,growth_site_tile_nums(q,which_site_for_tube), ...
		growth_site_rows(q,which_site_for_tube)) = 1;
    else
      filled_in_unnuc(r,growth_site_tile_nums_unnuc(r,which_site_for_tube), ...
		      growth_site_rows_unnuc(r,which_site_for_tube)) = 1;
    end
    tile_conc = tile_conc - orig_tile_conc/(tiles_in_volume*num_tile_types);
    % Check whether this on event completes a tile row
    %if (min(filled_in(q,:,growth_site_rows(which_site_for_tube))) > 0)
      % If so, might it allow us to move up the highest_row_complete
      % counter?
      % It will if every row between the current highest complete
      % one plus one and this one is complete
%      'filled in'
      %pause
      %can_move_up_complete = 1;
%      fprintf(1,'checking rows %d to %d\n',highest_row_complete+1, ...
%	      growth_site_rows(which_site));
      %pause;
      %for j=(highest_row_complete(q)+1):growth_site_rows(q,which_site_for_tube)
	%if min(filled_in(q,:,j)) == 0
	 % can_move_up_complete = 0;
	 % break;
	%end
      %end
      %fprintf('can move up complete check was %d.\n',can_move_up_complete);
      %if can_move_up_complete
	%highest_row_complete(q) = ...
	%    growth_site_rows(q,which_site_for_tube);
	% Might the rows beyond this one also have been
	% complete?  If so, we should continue to increment the
        % highest_row_complete up to the last complete row
	%pause
	%while min(filled_in(q,:,highest_row_complete(q)+1)) > 0
	%  highest_row_complete(q) = highest_row_complete(q) + 1;
	%end
    %  end
    %end
    if nuc_tube
      while min(filled_in(q,:,highest_row_complete(q)+1)) > 0
	highest_row_complete(q) = highest_row_complete(q) + 1;
      end
      % Check whether this on event initiates a new tile row
      % If so, it might allows us to move up the highest_row_empty
      % counter
      if (sum(filled_in(q,:,growth_site_rows(q,which_site_for_tube))) == 1)
	highest_row_empty(q) = growth_site_rows(q,which_site_for_tube) + 1;
      end
    else
      while min(filled_in_unnuc(r,:,highest_row_complete_unnuc(r)+1)) > 0
	highest_row_complete_unnuc(r) = highest_row_complete_unnuc(r) + 1;
      end
      % Check whether this on event initiates a new tile row
      % If so, it might allows us to move up the highest_row_empty
      % counter
      if (sum(filled_in_unnuc(r,:,growth_site_rows_unnuc(r,which_site_for_tube))) == 1)
	highest_row_empty_unnuc(r) = growth_site_rows_unnuc(r,which_site_for_tube) + 1;
      end
    end
  elseif which_rate < (total_on_rate + total_off_rate)
    % off rate
    which_rate = which_rate - total_on_rate;
    total_rate_seen = 0;
    nuc_tube = 1;
    for q=1:num_tubes
      total_rate_seen = total_rate_seen + num_removal_sites(q)*k_off ...
	  + num_removal_sites_first_row(q)*(k_off_one - k_off);
      if which_rate <= total_rate_seen
	break;
      end
    end
    if (q == num_tubes) && which_rate > total_rate_seen
      nuc_tube = 0;
      for r=1:num_unnuc_tubes
	total_rate_seen = total_rate_seen + num_removal_sites_unnuc(r)*k_off;
	if which_rate <= total_rate_seen 
	  break;
	end
      end
    end
    which_site_for_tube = 1;
    if nuc_tube
      total_rate_seen = total_rate_seen - num_removal_sites(q)*k_off ...
	  - num_removal_sites_first_row(q)*(k_off_one - k_off);
      for which_site_for_tube=1:num_removal_sites(q)
	if removal_site_rows(q,which_site_for_tube) > 1
	  total_rate_seen  = total_rate_seen + k_off;
	else
	  total_rate_seen  = total_rate_seen + k_off_one;
	end
	if which_rate <= total_rate_seen 
	  break;
	end
      end
    else
      total_rate_seen = total_rate_seen - num_removal_sites_unnuc(r)*k_off;
      for which_site_for_tube=1:num_removal_sites_unnuc(r)
	  total_rate_seen  = total_rate_seen + k_off;
	  if which_rate <= total_rate_seen 
	    break;
	  end
      end
    end
    if which_site_for_tube < 0
      which_site_for_tube
      r
      num_unnuc_tubes
      which_site
      total_sites_seen
      sum(num_removal_sites)
      sum(num_removal_sites_unnuc)
    end
    if nuc_tube
      filled_in(q,removal_site_tile_nums(q,which_site_for_tube), ...
		removal_site_rows(q,which_site_for_tube)) = 0;    
    else
      filled_in(r,removal_site_tile_nums_unnuc(r,which_site_for_tube), ...
		removal_site_rows_unnuc(r,which_site_for_tube)) = 0;    
    end
    tile_conc = tile_conc + orig_tile_conc/(tiles_in_volume* ...
					    num_tile_types);    
    
    if debug
      if nuc_tube 
	fprintf(1,'removing a site from tube %d site %d out of %f,%f, circum pos %d, row %d\n',...
		q,which_site_for_tube,which_rate,total_off_rate,...
		removal_site_tile_nums(q,which_site_for_tube),...
		removal_site_rows(q,which_site_for_tube));    
      else
	fprintf(1,'removing a site from unnuc tube %d site %d out of %f,%f, circum pos %d, row %d\n',...
		r,which_site_for_tube,which_rate,total_off_rate,...
		removal_site_tile_nums_unnuc(r,which_site_for_tube),...
		removal_site_rows_unnuc(r,which_site_for_tube));    
      end
    end
    % Did this site make a row empty now?   If so, we need to
    % change the highest_row_empty counter
    %if sum(filled_in(q,:,removal_site_rows(q,which_site_for_tube))) ...
    %	   == 0
      %'emptying'
    %  highest_row_empty(q) = removal_site_rows(q,which_site_for_tube);
    %end
    % Did we uncomplete a row?  If so we may need to move back the
    % complete marker to the previous row
    %if sum(filled_in(q,:,removal_site_rows(q,which_site_for_tube))) == ...
    %	 (tube_width - 1)
    %  highest_row_complete(q) = min(highest_row_complete(q), ...
%				 removal_site_rows(q,which_site_for_tube) - 1);
 %   end
     while highest_row_complete(q) > 1 && ...
	   min(filled_in(q,:,highest_row_complete(q)+1)) == 0
      highest_row_complete(q) = highest_row_complete(q) - 1;
     end
  elseif which_rate < (total_on_rate + total_off_rate + nucleation_rate)
    % Nucleation
    num_unnuc_tubes = num_unnuc_tubes + 1;
    nuc_tube = 0;
    r = num_unnuc_tubes;
  elseif which_rate < (total_on_rate + total_off_rate + nucleation_rate ...
		      + seed_unseed_joining_rate)
    'joining seeded and unseeded'
    % Joining between seeded and unseeded
    % Pick 2 random nanotubes to join
    tube_1 = randi(num_tubes);
    tube_2 = randi(num_unnuc_tubes);
    fprintf('Joining tubes seeded %d and unseeded %d.\n',tube_1,tube_2);
    % We'll remove tube 2 and copy its rows into a new tube, which
    % will become tube 1
    new_tube_length = highest_row_complete(tube_1) + ...
	highest_row_complete_unnuc(tube_2);
    store_old_tube = filled_in(tube_1,:,:);
    filled_in(tube_1,:,1:new_tube_length) = 1;
    row_to_copy_start = highest_row_complete(tube_1)+1;
    for p=row_to_copy_start:(row_to_copy_start+tube_width)
      filled_in(tube_1,:,p+highest_row_complete_unnuc(tube_2)) = store_old_tube(:,p);
    end
    highest_row_complete(tube_1) = highest_row_complete(tube_1) + highest_row_complete_unnuc(tube_2);
    highest_row_empty(tube_1) = highest_row_empty(tube_1) + highest_row_complete_unnuc(tube_2);
    growth_site_rows(tube_1,:) = growth_site_rows(tube_1,:) + ...
	highest_row_complete_unnuc(tube_2);
    
    % Now get rid of tube 2 by copying the last tube into this slot
    filled_in_unnuc(tube_2,:,:) = filled_in_unnuc(num_unnuc_tubes,:,:);
    growth_site_tile_nums_unnuc(tube_2,:) = ...
	growth_site_tile_nums_unnuc(num_unnuc_tubes,:);
    growth_site_rows_unnuc(tube_2,:) = growth_site_rows_unnuc(num_unnuc_tubes,:);
    highest_row_complete_unnuc(tube_2) = highest_row_complete_unnuc(num_unnuc_tubes);
    highest_row_empty_unnuc(tube_2) = highest_row_empty_unnuc(num_unnuc_tubes);
    num_growth_sites_unnuc(tube_2) = num_growth_sites_unnuc(num_unnuc_tubes);
    num_unnuc_tubes = num_unnuc_tubes - 1;
    fprintf('There are now %d seeded and %d unseeded.\n',num_tubes,num_unnuc_tubes);
  elseif which_rate < (total_on_rate + total_off_rate + nucleation_rate ...
		      + seed_unseed_joining_rate + ...
		       unseed_unseed_joining_rate)
    'unseeded unseeded joining'
    % Joining between 2 unseeded
    tube_1 = randi(num_unnuc_tubes);
    tube_2 = randi(num_unnuc_tubes);
    while tube_1 == tube_2
      tube_2 = randi(num_unnuc_tubes);
    end
    fprintf('Joining tubes unseeded %d and unseeded %d.\n',tube_1, ...
	    tube_2);

    % We'll remove tube 2 and copy its rows into a new tube, which
    % will become tube 1
    new_tube_length = highest_row_complete_unnuc(tube_1) + ...
	highest_row_complete_unnuc(tube_2);
    store_old_tube = filled_in_unnuc(tube_1,:,:);
    filled_in_unnuc(tube_1,:,1:new_tube_length) = 1;
    row_to_copy_start = highest_row_complete_unnuc(tube_1)+1;
    for p=row_to_copy_start:(row_to_copy_start+tube_width)
      filled_in_unnuc(tube_1,:,p+highest_row_complete_unnuc(tube_2)) = store_old_tube(:,p);
    end
    highest_row_complete_unnuc(tube_1) = highest_row_complete_unnuc(tube_1) + highest_row_complete_unnuc(tube_2);
    highest_row_empty_unnuc(tube_1) = highest_row_empty_unnuc(tube_1) + highest_row_complete_unnuc(tube_2);
    growth_site_rows_unnuc(tube_1,:) = growth_site_rows_unnuc(tube_1,:) + ...
	highest_row_complete_unnuc(tube_2);
    
    % Now get rid of tube 2 by copying the last tube into this slot
    filled_in_unnuc(tube_2,:,:) = filled_in_unnuc(num_unnuc_tubes,:,:);
    growth_site_tile_nums_unnuc(tube_2,:) = ...
	growth_site_tile_nums_unnuc(num_unnuc_tubes,:);
    growth_site_rows_unnuc(tube_2,:) = growth_site_rows_unnuc(num_unnuc_tubes,:);
    highest_row_complete_unnuc(tube_2) = highest_row_complete_unnuc(num_unnuc_tubes);
    highest_row_empty_unnuc(tube_2) = highest_row_empty_unnuc(num_unnuc_tubes);
    num_growth_sites_unnuc(tube_2) = num_growth_sites_unnuc(num_unnuc_tubes);
    num_unnuc_tubes = num_unnuc_tubes - 1;    
    fprintf('There are now %d seeded and %d unseeded.\n',num_tubes, ...
	    num_unnuc_tubes);
  elseif which_rate < (total_on_rate + total_off_rate + nucleation_rate ...
		       + seed_unseed_joining_rate + ...
		       unseed_unseed_joining_rate + ...
		       seeded_scission_rate)
    % Seeded nanotube scission
    % Pick which tube to split
    total_rows = sum(highest_row_complete) - num_tubes;
    row_to_break = randi(total_rows);
    tube_1 =0;
    rows_seen = 0;
    while rows_seen < row_to_break
      tube_1 = tube_1 + 1;
      rows_seen = rows_seen + highest_row_complete(tube_1) - 1;

    end
    if tube_1 > 1
      row_to_break = rows_seen - row_to_break + 1;
    end
    if debug
      fprintf('Breaking tube %d at row %d out of %d rows.\n',tube_1, ...
	      row_to_break,highest_row_complete(tube_1));
      % Now printout the structure of these tubes
      fprintf(1,['Original tube was seeded tube %d.  There are %d growth sites, %d removal sites.  '...
		 'highest row complete is %d, highest row empty is %d.\n'],...
	      tube_1,num_growth_sites(tube_1), num_removal_sites(tube_1), ...
	      highest_row_complete(tube_1), highest_row_empty(tube_1));
      fprintf(1,'Filled in rows are \n');
      for j=1:highest_row_empty(tube_1)
      if (mod(j,2) == 0)
	fprintf(1,' ');
      end
      for i=1:tube_width
	fprintf(1,'%d ',filled_in(tube_1,i,j));
      end
      fprintf(1,'\n');
      end
      fprintf(1,'\n\n');
      end
    
    % We'll copy the first rows into a temporary tube
    new_tube_length = row_to_break;
    new_second_tube_length = highest_row_complete(tube_1)- ...
	row_to_break ;
    %fprintf('NEW tubes are %d and %d rows long, respectively.\n',new_tube_length,new_second_tube_length);
    store_old_tube = filled_in(tube_1,:,:);

    % First create the new unseeded tube
    num_unnuc_tubes = num_unnuc_tubes + 1;
    filled_in_unnuc(num_unnuc_tubes,:,:) = 0;
    for i=1:new_second_tube_length
      for j=1:tube_width
	filled_in_unnuc(num_unnuc_tubes,j,i) = 1;
      end
    end
    row_to_copy_start = highest_row_complete(tube_1)+1;
    fprintf('Copying rows %d to %d, decreasing length by %d.\n',row_to_copy_start,row_to_copy_start+tube_width,new_second_tube_length);
    for p=row_to_copy_start:(row_to_copy_start+tube_width)
      filled_in_unnuc(num_unnuc_tubes,:,p-row_to_break) = store_old_tube(1,:,p);
    end

    max_ind = size(growth_site_rows_unnuc(num_unnuc_tubes,:),2);
    growth_site_rows_unnuc(num_unnuc_tubes,1:max_ind) = growth_site_rows(tube_1,1:max_ind) ...
	- row_to_break;
    growth_site_tile_nums_unnuc(num_unnuc_tubes,1:max_ind) = ...
	growth_site_tile_nums(tube_1,1:max_ind);
    highest_row_complete_unnuc(num_unnuc_tubes) = ...
	highest_row_complete(tube_1) - row_to_break;
    fprintf('new unseeded tube now has %d rows.\n',new_second_tube_length);
    highest_row_empty_unnuc(num_unnuc_tubes) = ...
	highest_row_empty(tube_1) - row_to_break;
    num_growth_sites_unnuc(num_unnuc_tubes) = ...
	num_growth_sites(tube_1);
    num_removal_sites_unnuc(num_unnuc_tubes) = num_removal_sites(tube_1);
    fprintf('Scission.  There are now %d seeded and %d unseeded.\n',num_tubes,num_unnuc_tubes);


    % Now shorten tube 1
    filled_in(tube_1,:,:) = 0;
    filled_in(tube_1,:,1:new_tube_length) = 1;
    fprintf(1,'Filled in rows are \n');

    row_to_copy_start = highest_row_complete(tube_1)+1;
    fprintf('Copying rows %d to %d, decreasing length by %d.\n',row_to_copy_start,row_to_copy_start+tube_width,new_second_tube_length);
    for p=row_to_copy_start:(row_to_copy_start+tube_width)
      filled_in(tube_1,:,p-new_second_tube_length) = store_old_tube(1,:,p);
    end
    highest_row_complete(tube_1) = highest_row_complete(tube_1) - new_second_tube_length;
    highest_row_empty(tube_1) = highest_row_empty(tube_1) - ...
	new_second_tube_length;
    growth_site_rows(tube_1,:) = growth_site_rows(tube_1,:) - ...
	new_second_tube_length;
    nuc_tube = 1;
    q = tube_1;
    r = num_unnuc_tubes;
    
    % Now print out the structure of these tubes
    fprintf(1,['Seeded tube %d.  There are %d growth sites, %d removal sites.  '...
	       'highest row complete is %d, highest row empty is %d.\n'],...
	    q,num_growth_sites(q), num_removal_sites(q), ...
	    highest_row_complete(q), highest_row_empty(q));
    if debug
      fprintf(1,'Filled in rows are \n');
      for j=1:highest_row_empty(q)
	if (mod(j,2) == 0)
	  fprintf(1,' ');
	end
	for i=1:tube_width
	  fprintf(1,'%d ',filled_in(q,i,j));
	end
	fprintf(1,'\n');
      end
      fprintf(1,'\n\n');
    end
    
    fprintf(1,['Unseeded tube %d.  There are %d growth sites, %d removal sites.  '...
	       'highest row complete is %d, highest row empty is %d.\n'],...
	    r,num_growth_sites_unnuc(r), num_removal_sites_unnuc(r), ...
	    highest_row_complete_unnuc(r), ...
	    highest_row_empty_unnuc(r));
    if debug
      fprintf(1,'Filled in rows are \n');
      for j=1:highest_row_empty_unnuc(r)
	if (mod(j,2) == 0)
	  fprintf(1,' ');
	end
      for i=1:tube_width
	fprintf(1,'%d ',filled_in_unnuc(r,i,j));
      end
      fprintf(1,'\n');
      end
      fprintf(1,'\n\n');
    end
    %pause;
    % End print out 
    nuc_tube = 0;
    r = num_unnuc_tubes;
  else  % Unseeded nanotube scission
    % Pick which tube to split
    total_rows = sum(highest_row_complete_unnuc(1:num_unnuc_tubes)) - num_unnuc_tubes;
    row_to_break = randi(total_rows);
    tube_1 =0;
    rows_seen = 0;
    while (rows_seen < row_to_break)       tube_1 = tube_1 + 1;
      rows_seen = rows_seen + highest_row_complete_unnuc(tube_1) - 1;
    end
    if tube_1 > 1
      row_to_break = rows_seen - row_to_break + 1;
    end
    fprintf('Breaking unseeded tube %d at row %d out of %d rows.\n',tube_1, ...
	    row_to_break,highest_row_complete_unnuc(tube_1));
    
    % We'll copy the first rows into a temporary tube
    new_tube_length = row_to_break;
    new_second_tube_length = highest_row_complete_unnuc(tube_1)- ...
	row_to_break ;
    fprintf('NEW tubes are %d and %d rows long, respectively.\n',new_tube_length,new_second_tube_length);
    store_old_tube = filled_in_unnuc(tube_1,:,:);

    % First create the new unseeded tube
    num_unnuc_tubes = num_unnuc_tubes + 1;
    filled_in_unnuc(num_unnuc_tubes,:,:) = 0;
    for i=1:new_second_tube_length
      for j=1:tube_width
	filled_in_unnuc(num_unnuc_tubes,j,i) = 1;
      end
    end
    row_to_copy_start = highest_row_complete_unnuc(tube_1)+1;
    fprintf('Copying rows %d to %d, decreasing length by %d.\n',row_to_copy_start,row_to_copy_start+tube_width,new_second_tube_length);
    for p=row_to_copy_start:(row_to_copy_start+tube_width)
      filled_in_unnuc(num_unnuc_tubes,:,p-row_to_break) = store_old_tube(1,:,p);
    end
    if debug
      num_unnuc_tubes
      tube_1
      size(growth_site_rows_unnuc(num_unnuc_tubes,:))
      size(growth_site_rows_unnuc(tube_1,:))
    end
    growth_site_rows_unnuc(num_unnuc_tubes,:) = growth_site_rows_unnuc(tube_1,:) ...
	- row_to_break;
    growth_site_tile_nums_unnuc(num_unnuc_tubes,:) = ...
	growth_site_tile_nums_unnuc(tube_1,:);
    highest_row_complete_unnuc(num_unnuc_tubes) = ...
	highest_row_complete_unnuc(tube_1) - row_to_break;
    fprintf('new unseeded tube now has %d rows.\n',new_second_tube_length);
    highest_row_empty_unnuc(num_unnuc_tubes) = ...
	highest_row_empty_unnuc(tube_1) - row_to_break;
    num_growth_sites_unnuc(num_unnuc_tubes) = ...
	num_growth_sites_unnuc(tube_1);
    num_removal_sites_unnuc(num_unnuc_tubes) = num_removal_sites_unnuc(tube_1);
    fprintf('Scission.  There are now %d seeded and %d unseeded.\n',num_tubes,num_unnuc_tubes);


    % Now shorten tube 1
    filled_in_unnuc(tube_1,:,:) = 0;
    filled_in_unnuc(tube_1,:,1:new_tube_length) = 1;
    fprintf(1,'Filled in rows are \n');

    row_to_copy_start = highest_row_complete_unnuc(tube_1)+1;
    fprintf('Copying rows %d to %d, decreasing length by %d.\n',row_to_copy_start,row_to_copy_start+tube_width,new_second_tube_length);
    for p=row_to_copy_start:(row_to_copy_start+tube_width)
      filled_in_unnuc(tube_1,:,p-new_second_tube_length) = store_old_tube(1,:,p);
    end
    highest_row_complete_unnuc(tube_1) = highest_row_complete_unnuc(tube_1) - new_second_tube_length;
    highest_row_empty_unnuc(tube_1) = highest_row_empty_unnuc(tube_1) - ...
	new_second_tube_length;
    growth_site_rows_unnuc(tube_1,:) = growth_site_rows_unnuc(tube_1,:) - ...
	new_second_tube_length;
    nuc_tube = 1;
    q = tube_1;
    r = num_unnuc_tubes;
    
    % Now print out the structure of these tubes
    fprintf(1,['Unseeded tube %d (first).  There are %d growth sites, %d removal sites.  '...
	       'highest row complete is %d, highest row empty is %d.\n'],...
	    q,num_growth_sites_unnuc(q), num_removal_sites_unnuc(q), ...
	    highest_row_complete_unnuc(q), highest_row_empty_unnuc(q));
    if debug
      fprintf(1,'Filled in rows are \n');
      for j=1:highest_row_empty(q)
        if (mod(j,2) == 0)
      	fprintf(1,' ');
        end
        for i=1:tube_width
      	fprintf(1,'%d ',filled_in(q,i,j));
        end
	fprintf(1,'\n');
      end
      fprintf(1,'\n\n');
    end
    
    fprintf(1,['Unseeded tube %d (second).  There are %d growth sites, %d removal sites.  '...
	       'highest row complete is %d, highest row empty is %d.\n'],...
	    r,num_growth_sites_unnuc(r), num_removal_sites_unnuc(r), ...
	    highest_row_complete_unnuc(r), ...
	    highest_row_empty_unnuc(r));
    if debug
      fprintf(1,'Filled in rows are \n');
      for j=1:highest_row_empty_unnuc(r)
	if (mod(j,2) == 0)
	  fprintf(1,' ');
	end
	for i=1:tube_width
	  fprintf(1,'%d ',filled_in_unnuc(r,i,j));
	end
	fprintf(1,'\n');
      end
      fprintf(1,'\n\n');
    end
    %pause;
    % End print out 
    nuc_tube = 0;
    r = num_unnuc_tubes;
  end
  % Update the "which rows filled" arrays for the assembly we
  % updated
  if nuc_tube
    num_growth_sites(q) = 0;
    num_removal_sites(q) = 0;
    num_removal_sites_first_row(q) = 0;
    for i=1:tube_width
      for j=(highest_row_complete(q)+1):highest_row_empty(q)
	% This is a growth site if it is empty and if it has at
        % least two filled neighbors.  
	if ((filled_in(q,i,j) == 0) && number_of_neighbors(q,i,j,filled_in,tube_width) ...
	    == 2)
	  num_growth_sites(q) = num_growth_sites(q) + 1;
	  growth_site_tile_nums(q,num_growth_sites(q)) = i;
	  growth_site_rows(q,num_growth_sites(q)) = j;
	end
	if ((filled_in(q,i,j) > 0) && number_of_neighbors(q,i,j,filled_in,tube_width) == 2)
	  num_removal_sites(q) = num_removal_sites(q) + 1;
	  if debug
	    j
	    pause;
	  end
	  if j == 2
	    num_removal_sites_first_row(q) = num_removal_sites_first_row(q) ...
		+ 1;
	  end
	  removal_site_tile_nums(q,num_removal_sites(q)) = i;
	  removal_site_rows(q,num_removal_sites(q)) = j;
	end
      end
    end
  else
    num_growth_sites_unnuc(r) = 0;
    num_removal_sites_unnuc(r) = 0;
    for i=1:tube_width
      for j=(highest_row_complete_unnuc(r)+1):highest_row_empty_unnuc(r)
	% This is a growth site if it is empty and if it has at
        % least two filled neighbors.  
	if ((filled_in_unnuc(r,i,j) == 0) && number_of_neighbors(r,i,j,filled_in_unnuc,tube_width) ...
	    == 2)
	  num_growth_sites_unnuc(r) = num_growth_sites_unnuc(r) + 1;
	  growth_site_tile_nums_unnuc(r,num_growth_sites_unnuc(r)) = i;
	  growth_site_rows_unnuc(r,num_growth_sites_unnuc(r)) = j;
	end
	if ((filled_in_unnuc(r,i,j) > 0) && number_of_neighbors(r,i,j,filled_in_unnuc,tube_width) == 2)
	  num_removal_sites_unnuc(r) = num_removal_sites_unnuc(r) + 1;
	  removal_site_tile_nums_unnuc(r,num_removal_sites_unnuc(r)) = i;
	  removal_site_rows_unnuc(r,num_removal_sites_unnuc(r)) = j;
	end
      end
    end
  end
  if debug 
    fprintf('num growth sites: ');
    num_growth_sites'
    fprintf('num removal sites: ');
    num_removal_sites'
    fprintf('removal site rows: ')
    removal_site_rows(q,:)
    fprintf('removal site tile nums: ')
    removal_site_tile_nums(q,:)
    fprintf(1,['Tube %d.  There are %d growth sites, %d removal sites.  '...
	       'highest row complete is %d, highest row empty is %d.\n'],...
	    q,num_growth_sites(q), num_removal_sites(q), ...
	    highest_row_complete(q), highest_row_empty(q));
    fprintf(1,'Filled in rows are \n');
  end
  if debug
    for j=(highest_row_complete(q)+1):highest_row_empty(q)
      if (mod(j,2) == 0)
	fprintf(1,' ');
      end
      for i=1:tube_width
        fprintf(1,'%d ',filled_in(q,i,j));
      end
      fprintf(1,'\n');
    end
    fprintf(1,'\n\n');
  end
  %for q=1:num_tubes
  %  while min(filled_in(q,:,highest_row_complete(q)+1)) > 0
  %    highest_row_complete(q) = highest_row_complete(q) + 1;
  %    fprintf('Found bad row -- pausing');
  %    pause;
  %  end
%end

  if rand() > 0.9999
    figure(1)
    fprintf('Total on rate is %f, total off rate is %f, total scission rate is %f, joining rate is %f.\n',total_on_rate,total_off_rate, ...
	     seeded_scission_rate+unseeded_scission_rate,seed_unseed_joining_rate+unseed_unseed_joining_rate);
    %long_enough_tubes = find(highest_row_complete*14.3/156 > 2.5);
    %hist(highest_row_complete(long_enough_tubes)*14.3/156,3:0.5:60)    
    if num_unnuc_tubes > 0
      all_rows = [highest_row_complete; highest_row_complete_unnuc(1:num_unnuc_tubes)];
    else
      all_rows = highest_row_complete;
    end
    hist(all_rows*14.3/1000,0:0.5:15)    
    axis([0 15 0 num_tubes+num_unnuc_tubes])
    title(['time is ' num2str(time) ' seconds']);
  end
  %figure(1)
  %imagesc(filled_in)
end


figure(1)
%long_enough_tubes = find(highest_row_complete*14.3/156 > 2.5);
%hist(highest_row_complete(long_enough_tubes)*14.3/156,3:0.5:60)
hist(highest_row_complete*14.3/156,0:0.5:60)    
title(['Nanotube length dist. after ' num2str(time/3600) ' hours'],...
'FontSize',20);
xlabel('Length (pixels)','FontSize',18);
ylabel('Number of tubes','FontSize',18);
set(gca,'FontSize',18);

%for k=1:num_tubes
%  fprintf('Tube %d, %d rows.\n',k,highest_row_complete(k));
%  for j=highest_row_complete(k):highest_row_empty(k)
%    if (mod(j,2) == 0)
%      fprintf(1,' ');
%    end
%    for i=1:tube_width
%      fprintf(1,'%d ',filled_in(k,i,j));
%    end
%    fprintf(1,'\n');
%  end
%  fprintf(1,'\n\n');
%end

function neighbors = number_of_neighbors(tube,circle_pos,row,filled_in,tube_width)
  % On the tube lattice ..
  if mod(circle_pos-1,tube_width) == 0
    mod_minus = tube_width;
  else
    mod_minus = mod(circle_pos-1,tube_width);
  end
  if mod(circle_pos+1,tube_width) == 0
    mod_plus = tube_width;
  else
    mod_plus = mod(circle_pos+1,tube_width);
  end
  %fprintf(1,'circle pos is %d, minus is %d, plus is %d.\n',circle_pos,mod_minus,mod_plus);
  if (mod(row,2)) 
    % if the row is odd
    % below and above neighbors are x-1,x
    neighbors = filled_in(tube,mod_minus,row+1) + ...
	filled_in(tube,circle_pos,row+1) ...
	+ filled_in(tube,mod_minus,row-1) + ...
	filled_in(tube,circle_pos,row-1);
  else
    % if the row is even
    % below and above neighbors are x,x+1
    neighbors = filled_in(tube,mod_plus,row+1) + ...
	filled_in(tube,circle_pos,row+1) + ...
	filled_in(tube,mod_plus,row-1) + ...
	filled_in(tube,circle_pos,row-1);
  end
